<?php
// $Id: distributed_search.admin.inc,v 1.1.2.2 2010/01/07 20:33:43 aufumy Exp $

/**
 * @file
 * Distributed search settings
 */

/**
 * Menu callback
 */
function distributed_search_admin() {
  $result = db_query('SELECT * FROM {distributed_search_source} ORDER BY title');
  $output = '<h3>'. t('Source overview') .'</h3>';

  $header = array(t('Title'), t('URL'), t('API key'), array('data' => t('Operations'), 'colspan' => ' 1'));
  $rows = array();
  while ($source = db_fetch_object($result)) {
    $rows[] = array(
      check_plain($source->title),
      check_url(url($source->url)), 
      check_plain($source->apikey),
      l(t('edit'), "admin/settings/distributed_search/edit/source/$source->sid")
    );
  }
  if (count($rows) == 0) {
    $rows[] = array(array('colspan' => 4, 'data' => t('No sources defined.')));
  }
  $output .= theme('table', $header, $rows);
  return $output;
}

/**
 * Generate a form to add/edit sources.
 */
function distributed_search_form_source($form_state, $nid = 0) {
  $edit = distributed_search_get_sources($nid);

  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#default_value' => $edit[$nid]->title,
    '#maxlength' => 64,
    '#description' => t('The name of the source; typically the name of the web site you syndicate content from.'),
    '#required' => TRUE,
  );
  $form['url'] = array(
    '#type' => 'textfield',
    '#title' => t('Description URL'),
    '#default_value' => $edit[$nid]->url,
    '#maxlength' => 255,
    '#description' => t('The fully-qualified URL of the Distributed Search client site.'),
    '#required' => TRUE,
  );
  $form['apikey'] = array(
    '#type' => 'textfield',
    '#title' => t('API key.'),
    '#default_value' => $edit[$nid]->apikey,
    '#maxlength' => 255,
    '#description' => t('The API key generated by the search client site, to connect to it.'),
    '#required' => TRUE,
  );

  $form['submit'] = array('#type' => 'submit', '#value' => t('Submit'));

  if ($edit[$nid]->sid) {
    $form['delete'] = array('#type' => 'submit', '#value' => t('Delete'));
    $form['sid'] = array('#type' => 'hidden', '#value' => $edit[$nid]->sid);
  }

  return $form;
}

/**
 * Validate distributed_search_form_source form submissions.
 */
function distributed_search_form_source_validate($form, &$form_state) {
  if ($form_state['values']['op'] == t('Submit')) {
    // Check for duplicate titles
    if (isset($form_state['values']['sid'])) {
      $result = db_query("SELECT title, url FROM {distributed_search_source} 
                          WHERE (title = '%s' OR url = '%s') AND sid <> %d", 
			  $form_state['values']['title'], $form_state['values']['url'], $form_state['values']['sid']);
    }
    else {
      $result = db_query("SELECT title, url FROM {distributed_search_source} 
                          WHERE title = '%s' OR url = '%s'", 
			  $form_state['values']['title'], $form_state['values']['url']);
    }
    while ($source = db_fetch_object($result)) {
      if (strcasecmp($source->title, $form_state['values']['title']) == 0) {
        form_set_error('title', t('A source named %source already exists. Please enter a unique title.', array('%source' => $form_state['values']['title'])));
      }
    }
    
    $discover = _check_http_request($form_state['values']['url']);
    if (! $discover['success']) {
      form_set_error('url', t('URL did not return a valid HTTP Response. The error was: %error', array('%error' => $discover['error'])));
    }
  }
}

/**
 * Process distributed_search_form_source form submissions.
 * @todo Add delete confirmation dialog.
 */
function distributed_search_form_source_submit($form, &$form_state) {
  if ($form_state['values']['op'] == t('Delete')) {
    $title = $form_state['values']['title'];
    // Unset the title:
    unset($form_state['values']['title']);
  }
  distributed_search_save_source($form_state['values']);
  if (isset($form_state['values']['sid'])) {
    if (isset($form_state['values']['title'])) {
      drupal_set_message(t('The source %source has been updated.',
			   array('%source' => $form_state['values']['title'])));
    }
    else {
      watchdog('dist_search', 
               'Source %source deleted.',
               array('%source' => $title),
               WATCHDOG_NOTICE,
               l(t('view'), 'admin/settings/distributed_search'));
      drupal_set_message(t('The source %source has been deleted.',
                           array('%source' => $title)));
    }
  }
  else {
    watchdog('dist_search', 
             'Source %source added.',
	     array('%source' => $form_state['values']['title']),
	     WATCHDOG_NOTICE,
	     l(t('view'), 'admin/settings/distributed_search'));
    drupal_set_message(t('The source %source has been added.', 
                         array('%source' => $form_state['values']['title'])));
  }
  $form_state['redirect'] = 'admin/settings/distributed_search';
}

/**
 * Add/edit/delete an aggregator source.
 */
function distributed_search_save_source($edit) {
  if (!empty($edit['sid']) && !empty($edit['title'])) {
    db_query("UPDATE {distributed_search_source} 
              SET title = '%s', url = '%s', apikey = '%s' WHERE sid = %d", 
	      $edit['title'], $edit['url'], $edit['apikey'], $edit['sid']);
  }
  else if (!empty($edit['sid'])) {
    db_query('DELETE FROM {distributed_search_source} WHERE sid = %d', $edit['sid']);
  }
  else if ($edit['title']) {
    db_query("INSERT INTO {distributed_search_source} 
              (title, url, apikey) VALUES ('%s', '%s', '%s')", 
	      $edit['title'], $edit['url'], $edit['apikey']);
    $edit['sid'] = db_last_insert_id('distributed_search_source', 'sid');
  }
}

